#!/usr/local/bin/perl

#
# html.pl
#
# EchoFile()
# FormatString()
# ParseForm()
# ProcessLine()
#

use File::Basename;

###############################################################################
# Procedure:   EchoFile
#
# Description: Reads the $fileName from $path and prints it after processing it
#              by doing variable substitution.
#
# Parameters:  $path - the filesystem path where the file is located.
#              $fileName - the filename of the file to be echoed.
#
# Return vals: 0 if file can't be opened, 1 otherwise.
###############################################################################
sub EchoFile {
    &LogItem("--> EchoFile()");
    local($path, $fileName) = @_;
    local($line, $lineCopy, $changes);

	if(!$CookieSet) {
		print "Content-type: text/html; charset=utf-8\n\n"; # Set Content Type
	}

	$path = "$ROOT_PATH$path";
	if($path[length($path)-1] ne "/") {
		$path = "$path/"; # Add / to end of path if it doesn't exist
	}

    # Open the template file and parse each line
    if (!open(FILE, "$path$fileName")) { 
        &LogItem("! open($path$fileName) failed: $!");
        return(0);
    }
    while ($line = <FILE>) {
        # Call our routine to do the variable substitution.
        $line = &ProcessLine( $line );

        # Print line depending on presence of variables existing
        if ($line) {
            print $line;
        }
    }
    close(FILE);
    &LogItem("<-- EchoFile()");
    return(1);
}

###############################################################################
# Procedure:   ProcessLine
#
# Description: Takes a line of text and returns it with any variables (denoted
#              by <<VAR_NAME>>) gone and in their place text from the %VAR hash
#              (i.e. <<VARIABLE_1>> would be replaced by $VAR{'VARIABLE_1'}).
#
# Parameters:  $line - The line to be processed
#
# Return vals: the processed line is returned.
###############################################################################
sub ProcessLine {
    local($line) = @_;
    local($lineCopy, $changes);

    # Initialize our variables
    $lineCopy = '';
    $changes = 0;

    # Search for variables in the line
    while ($line =~ /<<([^>]+)>>/) {

        # Store the part before the match so we do not repeat the
        # searching on this part.
        $lineCopy .= $`;

        # If something exists in the hash table, change line to its
        # value plus everything already in $line after it.
        # Otherwise, just set $line to find the next match.
        if (($VAR{$1} || ($VAR{$1} == 0))) {
            $line = $VAR{$1} . $';
            $changes++;
        }
        else {
            $line = $';
        }
    }

    # Set $line according to whether or not any matches were found
    $line = $lineCopy ? $lineCopy . $line : $line;

    return $line;
}

###############################################################################
# Procedure:   ParseForm
#
# Description: Takes the parameter string given by the CGI and places the
#              name and value pairs in the %FORM hash for use by the rest
#              of the script.
#
# Parameters:  (n/a)
#
# Return vals: 1 on success, 0 if an error occurred
###############################################################################
sub ParseForm {
    &LogItem("--> ParseForm()");
    local($name, $value, $pair, $buffer, @pairs);

    # Check for request method and handle appropriately
    if ($ENV{'REQUEST_METHOD'} eq 'GET') {
        @pairs = split(/&/, $ENV{'QUERY_STRING'});
    }
    elsif ($ENV{'REQUEST_METHOD'} eq 'POST') {
        read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
        @pairs = split(/&/, $buffer);
    }
    else {
        &LogItem("Bad request method: ($ENV{'REQUEST_METHOD'}).");
        return(0);
    }

    # Convert the data to its original format
    foreach $pair (@pairs) {
        ($name, $value) = split(/=/, $pair);

        $name =~ tr/+/ /;
        $name =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
        $name =~ s/\n//g;
        $value =~ tr/+/ /;
        $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
		if ($name ne 'DESCRIPTION') {
	        $value =~ s/\n//g;
		}

        # If they try to include server side includes, erase them, so they
        # arent a security risk if the HTML gets returned.  Another
        # security hole plugged up.
        $value =~ s/<!--(.|\n)*-->//g;

        # Store name/value pair in %CONFIG if the corresponding entry is defined
        if ($CONFIG{$name}) {
            $CONFIG{$name} .= ",$value";
        }
        elsif (defined($CONFIG{$name})) {
            $CONFIG{$name} = $value;
        }

        # Otherwise store in %FORM
        elsif ($FORM{$name}) {
            $FORM{$name} .= ",$value";
        }
        else {
            $FORM{$name} = $value;
        }
    }
    &LogItem("<-- ParseForm()");
    return(1);
}

###############################################################################
# Procedure:   FormatString
#
# Description: Formats a string for viewing in the HTML page.  Changes less-than
#              ('<') signs to '&lt;', '\n' to '<BR>', and hypertext buttons,
#              as implemented in Tivoli-Script, to media hyperlinks.
#
# Parameters:  $str - the string to be formatted.
#
# Return vals: The formatted string.
###############################################################################
sub FormatString {
    &LogItem("--> FormatString()");
    local($str) = @_;
    local($text, $cmd, $target, @options);
    local($pos, $len, $i, @letter);

    # Replace less-than signs ("<") with "&lt;" to avoid
    # confusion with HTML tags.
    $str =~ s/</&lt;/g;

    # Ensure proper indentation by inserting spaces
    # at the beginning of each line.
    $str =~ s/^( *)/'&nbsp;' x (length($1))/emg;

    # Add Hyperlinks for the hypertext buttons.
    # Programmer Note: The '?' characters are in the RegExp so that a minimum number of
    # characters are matched each time.
    while ($str =~ m/\[\[(.*?)\|[ \t]*(.*?)\]\]/) {
        $text = $1;
        ($cmd, $target, @options) = split(/[ \t]/, $2);
        if (($cmd eq 'SEARCH') || ($cmd eq 'RUN') || ($cmd eq 'PLAY')) {
            $str =~ s/\[\[.*?\]\]/$text/;
        }
        elsif ($cmd eq 'URL') {
            $str =~ s/\[\[.*?\]\]/<A HREF="$target" TARGET="_new">$text<\/A>/;
        }
        elsif ($cmd eq 'FILE') {
            $target =~ s/\.tx2$/\.txt/i;
			$target = basename($target); # Get file name
            $str =~ s/\[\[.*?\]\]/<A HREF="javascript:mediawindow('$HTML_PATH$MEDIA_PATH\/$target', '260', '250')">$text<\/A>/;
        }
        elsif ($cmd eq 'IMAGE') {
            $target =~ s/\.bmp$/\.$MEDIA_IMAGE_EXTENSION/i;
			$target = basename($target); # Get file name
            $str =~ s/\[\[.*?\]\]/<A HREF="javascript:mediawindow('$HTML_PATH$MEDIA_PATH\/$target', '260', '250')">$text<\/A>/;
        }
        else {
            # We don't know what this is, so we'll just remove it.
            $str =~ s/\[\[.*?\]\]//;
        }
    }

    # Change newlines into HTML breaks.
    $str =~ s/\n\r/<BR>/g;
    $str =~ s/\n/<BR>/g;

    &LogItem("<-- FormatString()");
    return $str;
}

###############################################################################
# Procedure:   FormatLinkedString
#
# Description: Formats a string for viewing in the HTML page.  This function is
#              used whenever the string is part of a link, so that hypertext buttons
#              are not changed to links.
#
# Parameters:  $str - the string to be formatted.
#
# Return vals: The formatted string.
###############################################################################
sub FormatLinkedString {
    &LogItem("--> FormatLinkedString()");
    my($str) = @_;
    my ($text, $etc);

    # Replace less-than signs ("<") with "&lt;" to avoid
    # confusion with HTML tags.
    $str =~ s/</&lt;/g;

    while ($str =~ m/\[\[(.*?)\|[ \t]*(.*?)\]\]/) {
        $text = $1;
        $str =~ s/\[\[.*?\]\]/$text/;
    }

    &LogItem("<-- FormatLinkedString()");
    return $str;
}

###############################################################################
#
# Following are simple utility functions for outputting common HTML symbols.
#
###############################################################################

sub Bold {
    my($string) = @_;

	return "<B>$string</B>";
}

sub Center {
    my($string) = @_;

    return "<CENTER>$string</CENTER>";
}

sub Link {
    my ($string, $href, $target) = @_;
    my ($str);

    $str = "<A HREF=\"$href\"";
    if ($target) {
        $str .= " TARGET=\"$target\"";
    }
    $str .= ">$string</A>";

    return $str;
}

sub BR {
    return "<BR>";
}

sub TR {
    my ($string) = @_;

    return "<TR>$string</TR>\n";
}

sub TD {
    my ($string) = @_;

    return "<TD>$string</TD>";
}

sub UL {
    my ($string, $ret);

    $ret = "<UL>\n";
    foreach $string (@_) {
        $ret .= "\t<LI>$string\n";
    }
    $ret .= "</UL>\n";
}

sub InfoStyle {
    my ($string, $color) = @_;

    return "<CENTER><FONT SIZE=4 COLOR=\"$color\">$string</FONT></CENTER>";
}

sub Hidden {
    my ($key, $value) = @_;

    return "<INPUT TYPE=\"HIDDEN\" NAME=\"$key\" VALUE=\"$value\">\n";
}

# Needed for reuire statement in Main

1;
